# General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
# base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
# The model can simulate OLEDs, Perovskite cells, and OFETs.
# 
# Copyright (C) 2008-2020 Roderick C. I. MacKenzie
# All rights reserved.
# 
# https://www.gpvdm.com
# r.c.i.mackenzie at googlemail.com
# 
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the GPVDM nor the
#       names of its contributors may be used to endorse or promote products
#       derived from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
# DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


import sys
import pygtk
inp_dir='/usr/share/opvdm/'
gui_dir='/usr/share/opvdm/gui/'
lib_dir='/usr/lib64/opvdm/'
sys.path.append('./gui/')
sys.path.append(lib_dir)

import os
import shutil
import signal
import subprocess
from util import replace
from util import get_token_value

import os, fnmatch
import stat

def to_exp(data):
	ret=data
	if (ret.count('e')!=0):
		data=float(ret)
		ret="%1.1e" % data
		a,b=ret.split('e')
		ret=a+"\\e{"+b+"}"
	return ret

def to_nm(data):
	ret=str(float(data)*1e9)
	return ret

def to_mev(data):
	temp=float(data)
	temp=temp*1000.0
	ret=str(temp)
	if (ret.count('e')!=0):
		a,b=ret.split('e')
		ret=a+"\\e{"+b+"}"
	return ret

def export_as(mode,output):
	ext= os.path.splitext(output)[1]
	line=""
	if (ext==".pdf") or (ext==".jpg") or (ext==".tex"):
		line=line+"\\documentclass{article}\n"
		line=line+"\\providecommand{\\e}[1]{\\ensuremath{\\times 10^{#1}}}\n"
		line=line+"\\begin{document}\n"
		line=line+"\\pagenumbering{gobble}\n"
		line=line+"\n"
		line=line+"\\begin{table}[H]\n"
		line=line+"\\begin{center}\n"
		line=line+"  \\begin{tabular}{lll}\n"
		line=line+"  \\hline\n"
		line=line+"  Parameter & label & description \\\\\n"
		line=line+"  \\hline\n"


		line=line+"Density of electron trap states & $"+to_exp(get_token_value("./dos0.inp", "#Ntrape"))+"$ & $m^{-3} eV^{-1}$ \\\\\n"
		line=line+"Density of hole trap states & $"+to_exp(get_token_value("./dos0.inp", "#Ntraph"))+"$ & $m^{-3} eV^{-1}$ \\\\\n"
		line=line+"Free electron mobility & $"+to_exp(get_token_value("./dos0.inp", "#mueffe"))+"$ & $m^{2} V^{-1} s^{-1}$ \\\\\n"
		line=line+"Free hole mobility & $"+to_exp(get_token_value("./dos0.inp", "#mueffh"))+"$ & $m^{2} V^{-1} s^{-1}$ \\\\\n"
		line=line+"Free electron-trapped electron capture cross section & $"+to_exp(get_token_value("./dos0.inp", "#srhsigman_e"))+"$ & $m^{-2}$ \\\\\n"
		line=line+"Trapped electron-free hole capture cross section & $"+to_exp(get_token_value("./dos0.inp", "#srhsigmap_e"))+"$ & $m^{-2}$ \\\\\n"
		line=line+"Free electron-trapped hole capture cross section & $"+to_exp(get_token_value("./dos0.inp", "#srhsigman_h"))+"$ & $m^{-2}$ \\\\\n"
		line=line+"Free hole-trapped hole capture cross section & $"+to_exp(get_token_value("./dos0.inp", "#srhsigmap_h"))+"$ & $m^{-2} $ \\\\\n"
		line=line+"Density of carriers on the majority p-contact & $"+to_exp(get_token_value("./device.inp", "#lcharge"))+"$ & $m^{-3} $ \\\\\n"
		line=line+"Density of carriers on the majority n-contact & $"+to_exp(get_token_value("./device.inp", "#rcharge"))+"$ & $m^{-3} $ \\\\\n"
		line=line+"Effective density of free electron states & $"+to_exp(get_token_value("./dos0.inp", "#Nc"))+"$ & $m^{-3}$ \\\\\n"
		line=line+"Effective density of free hole states & $"+to_exp(get_token_value("./dos0.inp", "#Nv"))+"$ & $m^{-3} $ \\\\\n"
		line=line+"Electron tail slope & $"+to_mev(get_token_value("./dos0.inp", "#Etrape"))+"$ & $meV$ \\\\\n"
		line=line+"Hole tail slope & $"+to_mev(get_token_value("./dos0.inp", "#Etraph"))+"$ & $meV$ \\\\\n"
		line=line+"Relative permittivity & $"+get_token_value("./dos0.inp", "#epsilonr")+"$ & $m^{-3} eV^{-1}$ \\\\\n"
		line=line+"LUMO mobility edge & $"+get_token_value("./dos0.inp", "#Xi")+"$ & $eV$ \\\\\n"
		line=line+"Effective band gap & $"+to_exp(get_token_value("./dos0.inp", "#Eg"))+"$ & $eV$ \\\\\n"
		line=line+"Thermal velocity & $"+to_exp(get_token_value("./dos0.inp", "#srhvth_e"))+"$ & $m s^{-1}$ \\\\\n"
		line=line+"Shunt resistance & $"+to_exp(get_token_value("./device.inp", "#Rshunt"))+"$ & $\\Omega$ \\\\\n"
		line=line+"External series resistance & $"+to_exp(get_token_value("./device.inp", "#Rcontact"))+"$ & $\\Omega$ \\\\\n"
		line=line+"Cell thickness & $"+to_nm(get_token_value("./device_epitaxy.inp", "#layer1"))+"$ & $nm$ \\\\\n"
		line=line+"Doping & $"+to_exp(get_token_value("./dos0.inp", "#doping"))+"$ & $m^{-3}$ \\\\\n"
		line=line+"Photon generation efficiency & $"+to_exp(get_token_value("./light.inp", "#Dphotoneff"))+"$ & au \\\\\n"

		line=line+"  \\hline\n"
		line=line+"\\end{tabular}\n"
		line=line+"\\end{center}\n"
		line=line+"\\caption{Device data}\n"
		line=line+"\\end{table}\n"
		line=line+"\n"

		if (mode=="tof"):
			line=line+"\\begin{table}[H]\n"
			line=line+"\\begin{center}\n"
			line=line+"  \\begin{tabular}{lll}\n"
			line=line+"  \\hline\n"
			line=line+"  Parameter & label & description \\\\\n"
			line=line+"  \\hline\n"


			line=line+"Stop time & $"+to_exp(get_token_value("./tof.inp", "#tofstop"))+"$ & $s$ \\\\\n"
			line=line+"Laser off time & $"+to_exp(get_token_value("./tof.inp", "#toflaseroff"))+"$ & $s$ \\\\\n"
			line=line+"dt & $"+to_exp(get_token_value("./tof.inp", "#dt"))+"$ & $s$ \\\\\n"
			line=line+"dt multiplyer & $"+to_exp(get_token_value("./tof.inp", "#tofdtmull"))+"$ & $au$ \\\\\n"
			line=line+"ToF voltage & $"+to_exp(get_token_value("./tof.inp", "#tof_voltage"))+"$ & $V$ \\\\\n"
			line=line+"ToF R_{short} & $"+to_exp(get_token_value("./tof.inp", "#tof_Rshort"))+"$ & $\\Omega$ \\\\\n"

			line=line+"  \\hline\n"
			line=line+"\\end{tabular}\n"
			line=line+"\\end{center}\n"
			line=line+"\\caption{ToF data}\n"
			line=line+"\\end{table}\n"
			line=line+"\n"

		line=line+"\\end{document}\n"

		text_file = open("doc.tex", "w")
		text_file.write(line)
		text_file.close()

		if (ext==".pdf"):
			os.system("latex -interaction=batchmode doc")
			os.system("dvipdf doc.dvi")
			os.system("mv doc.pdf "+output)

		if (ext==".jpg"):
			os.system("latex -interaction=batchmode doc")
			os.system("convert -trim -bordercolor White -border 20x10 +repage -density 300 doc.dvi doc.jpg")
			os.system("mv doc.jpg "+output)

		if (ext==".tex"):
			os.system("mv doc.tex "+output)

	elif (ext==".gz"):
		cmd = 'tar -czvf '+output+' ./*.inp ./*.dat '
		os.system(cmd)



