// General-purpose Photovoltaic Device Model gpvdm.com - a drift diffusion
// base/Shockley-Read-Hall model for 1st, 2nd and 3rd generation solarcells.
// The model can simulate OLEDs, Perovskite cells, and OFETs.
// 
// Copyright (C) 2008-2020 Roderick C. I. MacKenzie
// All rights reserved.
// 
// https://www.gpvdm.com
// r.c.i.mackenzie at googlemail.com
// 
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//     * Redistributions of source code must retain the above copyright
//       notice, this list of conditions and the following disclaimer.
//     * Redistributions in binary form must reproduce the above copyright
//       notice, this list of conditions and the following disclaimer in the
//       documentation and/or other materials provided with the distribution.
//     * Neither the name of the GPVDM nor the
//       names of its contributors may be used to endorse or promote products
//       derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL Roderick C. I. MacKenzie BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <math.h>
#include <time.h>
#include <dirent.h>
#include <util.h>
#include <pwd.h>
#include <unistd.h>
#include "sim.h"
#include "dos.h"
#include "dump.h"
#include "complex_solver.h"
#include "elec_plugins.h"
#include "ntricks.h"


struct device cell;

int run_simulation(char *runpath)
{

printf("Run_simulation\n");

cell.newton_cur=FALSE;

if (strcmp(runpath,"")!=0) strcpy(cell.outputpath,runpath);
int i;

cell.kTq=(300.0*kb/q);
printf("Load config\n");
load_config("device.inp",&cell);

printf("Loading DoS for %d layers\n",cell.mat_layers);
char tempn[100];
char tempp[100];
i=0;
#ifdef enable_multi_layers
for (i=0;i<cell.mat_layers;i++)
{
#endif
	dos_init(i);
	printf("Load DoS %d/%d\n",i,cell.mat_layers);
	sprintf(tempn,"dosn%d.dat",i);
	sprintf(tempp,"dosp%d.dat",i);
	load_dos(&cell,tempn,tempp,i);
#ifdef enable_multi_layers
}
#endif
device_init(&cell);


if (get_dump_status(dump_print_converge)==TRUE)
{
cell.converge = fopena(cell.outputpath,"./converge.dat","w");
fclose(cell.converge);

cell.tconverge=fopena(cell.outputpath,"./tconverge.dat","w");
fclose(cell.tconverge);
}


int curlayer=0;
double end=cell.mat.l[0].height;

for (i=0;i<cell.ymeshpoints;i++)
{
	if (cell.ymesh[i]>=end)
	{
		curlayer++;
		end+=cell.mat.l[curlayer].height;
	}
	cell.imat[i]=curlayer;
	cell.Nad[i]=get_dos_doping(cell.imat[i]);

}
//getchar();

init_mat_arrays(&cell);

//getchar();
	for (i=0;i<cell.ymeshpoints;i++)
	{
		cell.phi[i]=0.0;
		cell.R[i]=0.0;
		cell.n[i]=0.0;
	}

cell.C=cell.xlen*cell.zlen*epsilon0*cell.epsilonr[0]/(cell.ylen+cell.other_layers);
if (get_dump_status(dump_print_text)==TRUE) printf("C=%le\n",cell.C);
cell.A=cell.xlen*cell.zlen;
cell.Vol=cell.xlen*cell.zlen*cell.ylen;

light_init(&cell.mylight,&cell);
light_set_dx(&cell.mylight,cell.ymesh[1]-cell.ymesh[0]);
light_load_config(&cell.mylight,cell.outputpath);
if (get_dump_status(dump_iodump)==FALSE) set_dump_status(dump_optics,FALSE);

//update_arrays(&cell);

cell.Vapplied=0.0;
get_initial(&cell);

remesh_shrink(&cell);

solve_pos(&cell);

remove_dir(cell.outputpath,"snapshots");
remove_dir(cell.outputpath,"light_dump");


time_init(&cell);

cell.N=0;
cell.M=0;

solver_realloc(&cell);

if (cell.onlypos==TRUE)
{
	device_free(&cell);
	return 0;
}

plot_open(&cell);

cell.stop=FALSE;

#ifdef enable_time
cell.go_time=FALSE;
#endif

plot_now(&cell,"plot");
//set_solver_dump_every_matrix(1);
find_n0(&cell);
//set_solver_dump_every_matrix(0);
draw_gaus(&cell);


#include "run_list.c"


device_free(&cell);

plot_close(&cell);

for (i=0;i<cell.mat_layers;i++)
{
	dos_free(i);
}

solver_free_memory(&cell);
light_free(&cell.mylight);

return cell.odes;
}

char *sim_output_path()
{
return cell.outputpath;
}

